@@ -1,7 +1,7 @@
 // samplv1_ramp.h
 //
 /****************************************************************************
-   Copyright (C) 2012, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -74,7 +74,7 @@ public:
 			reset();
 			m_frames = nframes;
 			const uint32_t MIN_FRAMES = 32;
-			if (m_frames > MIN_FRAMES)
+			if (m_frames < MIN_FRAMES)
 				m_frames = MIN_FRAMES;
 			for (uint16_t i = 0; i < m_nvalues; ++i)
 				m_delta[i] = (m_value1[i] - m_value0[i]) / float(m_frames);
@@ -130,12 +130,17 @@ protected:
 
 	virtual float evaluate(uint16_t)
 	{
-		if (m_param1)
-			m_param1_v = *m_param1;
+		update();
 
 		return m_param1_v;
 	}
 
+	void update()
+	{
+		if (m_param1)
+			m_param1_v = *m_param1;
+	}
+
 	float *m_param1;
 	float  m_param1_v;
 };
@@ -169,12 +174,19 @@ protected:
 
 	virtual float evaluate(uint16_t i)
 	{
-		if (m_param2)
-			m_param2_v = *m_param2;
+		update();
 
 		return samplv1_ramp1::evaluate(i) * m_param2_v;
 	}
 
+	void update()
+	{
+		samplv1_ramp1::update();
+
+		if (m_param2)
+			m_param2_v = *m_param2;
+	}
+
 	float *m_param2;
 	float  m_param2_v;
 };
@@ -208,12 +220,19 @@ protected:
 
 	virtual float evaluate(uint16_t i)
 	{
-		if (m_param3)
-			m_param3_v = *m_param3;
+		update();
 
 		return samplv1_ramp2::evaluate(i) * m_param3_v;
 	}
 
+	void update()
+	{
+		samplv1_ramp2::update();
+
+		if (m_param3)
+			m_param3_v = *m_param3;
+	}
+
 	float *m_param3;
 	float  m_param3_v;
 };
@@ -249,12 +268,19 @@ protected:
 
 	virtual float evaluate(uint16_t i)
 	{
-		if (m_param4)
-			m_param4_v = *m_param4;
+		update();
 
 		return samplv1_ramp3::evaluate(i) * m_param4_v;
 	}
 
+	void update()
+	{
+		samplv1_ramp3::update();
+
+		if (m_param4)
+			m_param4_v = *m_param4;
+	}
+
 	float *m_param4;
 	float  m_param4_v;
 };
