@@ -144,6 +144,8 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	m_ui.Pha1WetKnob->setSpecialValueText(sOff);
 	m_ui.Del1WetKnob->setSpecialValueText(sOff);
 
+	m_ui.Del1BpmKnob->setSpecialValueText(tr("Auto"));
+
 	// GEN note limits.
 	m_ui.Gen1SampleKnob->setMinimum(0.0f);
 	m_ui.Gen1SampleKnob->setMaximum(127.0f);
@@ -394,6 +396,10 @@ samplv1widget::samplv1widget ( QWidget *pParent, Qt::WindowFlags wflags )
 	setParamKnob(samplv1::DEL1_FEEDB, m_ui.Del1FeedbKnob);
 	setParamKnob(samplv1::DEL1_BPM,   m_ui.Del1BpmKnob);
 
+	QObject::connect(m_ui.Del1BpmKnob,
+		SIGNAL(valueChanged(float)),
+		SLOT(bpmSyncChanged()));
+
 	// Dynamics
 	setParamKnob(samplv1::DYN1_COMPRESS, m_ui.Dyn1CompressKnob);
 	setParamKnob(samplv1::DYN1_LIMITER,  m_ui.Dyn1LimiterKnob);
@@ -498,8 +504,21 @@ void samplv1widget::setParamValue ( samplv1::ParamIndex index, float fValue )
 
 float samplv1widget::paramValue ( samplv1::ParamIndex index ) const
 {
+	float fParamValue = 0.0f;
+
 	samplv1widget_knob *pKnob = paramKnob(index);
-	return (pKnob ? pKnob->value() : 0.0f);
+	if (pKnob) {
+		fParamValue = pKnob->value();
+	} else {
+		samplv1 *pSampl = instance();
+		if (pSampl) {
+			const float *pParamPort = pSampl->paramPort(index);
+			if (pParamPort)
+				fParamValue = *pParamPort;
+		}
+	}
+
+	return fParamValue;
 }
 
 
@@ -541,8 +560,12 @@ void samplv1widget::updateParamEx ( samplv1::ParamIndex index, float fValue )
 		m_ui.Gen1Sample->setLoopEnd(pSampl->loopEnd());
 		m_ui.Gen1LoopRangeFrame->setEnabled(bLoop);
 		updateSampleLoop(pSampl->sample());
-		// Fall thru...
+		break;
 	}
+	case samplv1::DEL1_BPMSYNC:
+		if (fValue > 0.0f)
+			m_ui.Del1BpmKnob->setValue(0.0f);
+		// Fall thru...
 	default:
 		break;
 	}
@@ -765,6 +788,10 @@ void samplv1widget::savePreset ( const QString& sFilename )
 #endif
 	const QString& sPreset = QFileInfo(sFilename).completeBaseName();
 
+	const QFileInfo fi(sFilename);
+	const QDir currentDir(QDir::current());
+	QDir::setCurrent(fi.absolutePath());
+
 	QDomDocument doc(SAMPLV1_TITLE);
 	QDomElement ePreset = doc.createElement("preset");
 	ePreset.setAttribute("name", sPreset);
@@ -795,6 +822,8 @@ void samplv1widget::savePreset ( const QString& sFilename )
 
 	m_ui.StatusBar->showMessage(tr("Save preset: %1").arg(sPreset), 5000);
 	updateDirtyPreset(false);
+
+	QDir::setCurrent(currentDir.absolutePath());
 }
 
 
@@ -871,7 +900,8 @@ void samplv1widget::saveSamples (
 
 	QDomElement eFilename = doc.createElement("filename");
 	eFilename.appendChild(doc.createTextNode(
-		QString::fromUtf8(pszSampleFile)));
+		QDir::current().relativeFilePath(
+			QString::fromUtf8(pszSampleFile))));
 	eSample.appendChild(eFilename);
 
 	uint32_t iLoopStart = pSampl->loopStart();
@@ -1077,6 +1107,29 @@ void samplv1widget::updateSampleLoop ( samplv1_sample *pSample, bool bDirty )
 }
 
 
+// Delay BPM change.
+void samplv1widget::bpmSyncChanged (void)
+{
+	if (m_iUpdate > 0)
+		return;
+
+	++m_iUpdate;
+	samplv1 *pSampl = instance();
+	if (pSampl) {
+		float *pBpmSync = pSampl->paramPort(samplv1::DEL1_BPMSYNC);
+		if (pBpmSync) {
+			const bool bBpmSync0
+				= (*pBpmSync > 0.0f);
+			const bool bBpmSync1
+				= (m_ui.Del1BpmKnob->minimum() >= m_ui.Del1BpmKnob->value());
+			if ((bBpmSync1 && !bBpmSync0) || (!bBpmSync1 && bBpmSync0))
+				*pBpmSync = (bBpmSync1 ? 1.0f : 0.0f);
+		}
+	}
+	--m_iUpdate;
+}
+
+
 // Sample context menu.
 void samplv1widget::contextMenuRequest ( const QPoint& pos )
 {
