@@ -22,17 +22,7 @@
 #ifndef __samplv1widget_config_h
 #define __samplv1widget_config_h
 
-#include "config.h"
-
-#define SAMPLV1_TITLE	PACKAGE_NAME
-#define SAMPLV1_VERSION	PACKAGE_VERSION
-
-#define SAMPLV1_SUBTITLE     "an old-school polyphonic sampler."
-#define SAMPLV1_WEBSITE      "http://samplv1.sourceforge.net"
-#define SAMPLV1_COPYRIGHT    "Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved."
-
-#define SAMPLV1_DOMAIN	"rncbc.org"
-
+#include "samplv1_config.h"
 
 #include <QSettings>
 #include <QStringList>
