@@ -1,7 +1,7 @@
 // samplv1widget_jack.h
 //
 /****************************************************************************
-   Copyright (C) 2012, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -28,6 +28,10 @@
 // Forward decls.
 class samplv1_jack;
 
+#ifdef CONFIG_NSM
+class samplv1_nsm;
+#endif
+
 
 //-------------------------------------------------------------------------
 // samplv1widget_jack - decl.
@@ -62,21 +66,50 @@ protected slots:
 
 #endif	// CONFIG_JACK_SESSION
 
+#ifdef CONFIG_NSM
+
+protected slots:
+
+	// NSM callback slots.
+	void openSession();
+	void saveSession();
+
+	void hideSession();
+	void showSession();
+
+#endif	// CONFIG_NSM
+
 protected:
 
 	// Synth engine accessor.
 	samplv1 *instance() const;
 
-	// Param methods.
+	// Param port method.
 	void updateParam(samplv1::ParamIndex index, float fValue) const;
 
+	// Dirty flag method.
+	void updateDirtyPreset(bool bDirtyPreset);
+
 	// Application close.
 	void closeEvent(QCloseEvent *pCloseEvent);
 
+#ifdef CONFIG_NSM
+
+	// Optional GUI handlers.
+	void showEvent(QShowEvent *pShowEvent);
+	void hideEvent(QHideEvent *pHideEvent);
+
+#endif	// CONFIG_NSM
+
 private:
 
 	// Instance variables.
 	samplv1_jack *m_pSampl;
+
+#ifdef CONFIG_NSM
+	samplv1_nsm *m_pNsmClient;
+	bool m_bNsmDirty;
+#endif
 };
 
 
