@@ -1,7 +1,7 @@
 // samplv1widget_wave.cpp
 //
 /****************************************************************************
-   Copyright (C) 2012, rncbc aka Rui Nuno Capela. All rights reserved.
+   Copyright (C) 2012-2013, rncbc aka Rui Nuno Capela. All rights reserved.
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -120,8 +120,9 @@ void samplv1widget_wave::paintEvent ( QPaintEvent *pPaintEvent )
 	const bool bDark = (pal.window().color().value() < 0x7f);
 	const QColor& rgbLite = (isEnabled()
 		? (bDark ? Qt::darkYellow : Qt::yellow) : pal.mid().color());
+    const QColor& rgbDark = pal.window().color().darker(180);
 
-	painter.fillRect(rect, pal.dark().color());
+	painter.fillRect(rect, rgbDark);
 
 	painter.setPen(bDark ? pal.mid().color() : pal.midlight().color());
 	painter.drawLine(w2, 0, w2, h);
